#! /usr/bin/perl
use strict;

# read file
my @lines = `cat queries;`;

# graph
my $scale;
my $maxsize = 100;
my $barlength;

# process file
my %ips;
my %domains;
my %subdomains;
my %queries;
my %queriesperday;
my %maxqueriesperday = 0;
my %times;
my %hourofday;
my $lastquery;
my $lastip;
foreach (@lines) {
	my ($ip,$time,$duration,$query) = split('\t');

	$query =~ s/^[^\w\.\/-]+//;
	$query =~ s/[^\w\.\/-]+$//;
	if (($query ne $lastquery || $ip ne $lastip) && ($ip =~ m/137\.224/))  { # reduce double counting, and only show local net.
#	if ($query ne $lastquery || $ip ne $lastip) { # reduce double counting

		$ips{$ip}++; 				# count number of searches per ip.

		my $subdomain = $ip;
		$subdomain =~ s/\.\d+$//;
		$subdomains{$subdomain}++;		# count number of searches per subdomain.

		my $domain = $subdomain;
		$domain =~ s/\.\d+$//;
		$domains{$domain}++;			# count number of searches per domain.

		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime($time);
		$mday = sprintf("%02d",$mday);
		$mon = sprintf("%02d",$mon+1);
		$year += 1900;
		$queriesperday{"$year$mon$mday"}++;	# count number of searches per day.

		$times{$duration}++;			# count number of searches per duration of search.

		my @qs = split(/[^\w\.\/-]+/, $query);
		foreach (@qs) {
			$queries{$_}++;			# count number of searches per searchword.
		}
		$lastquery = $query;
		$lastip = $ip;

		my ($tsec, $tmin, $thour, $tday, $tmon, $tyear, $wday, $yday, $isdst) = localtime($time);
		$hourofday{$thour}++;			# count number of searches per hour of day.
	}
}

my $head=<<THEEND;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-US"
lang="en-US">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=us-ascii" />
<title>Statistics.</title>
<link rel="stylesheet" type="text/css" href="/zoek/style.css" />
</head>
<body>
<div class="content">
<table>
THEEND

my $tail=<<THEEND;
</table>
</div>
</body>
</html>
THEEND

# print ip and # of times it accessed the program
$scale = $maxsize/max_hash(\%ips);
open(FH, "> topips.html");
print FH $head;
print FH "<tr><th>ip</th><th>searches</th><th>graph</th></tr>\n";
my $i = 0;

foreach (sort {$ips{$b} <=> $ips{$a}} keys (%ips)) {
	if ($i < 100) {
		print FH "
			<tr>
				<td>$_</td>
				<td>".$ips{$_}."</td>
				<td><img class=\"imgleft\" src=\"punt.png\" height=\"10\" width=\"".$ips{$_}*$scale."\"></img></td>
			</tr>\n";
		$i++;
	}
}
print FH $tail;
close(FH);

# print subdomain and # of times it accessed the program
$scale = $maxsize/max_hash(\%subdomains);
open(FH, "> topsubdomains.html");
print FH $head;
print FH "<tr><th>subdomain</th><th>searches</th><th>graph</th></tr>\n";my $i = 0;
foreach (sort {$subdomains{$b} <=> $subdomains{$a}} keys (%subdomains)) {
        if ($i < 100) {
                print FH "<tr><td>$_</td><td>".$subdomains{$_}."</td>
		<td><img class=\"imgleft\"  src=\"punt.png\" height=\"10\" width=\"".$subdomains{$_}*$scale."\"></img></td></tr>\n";
                $i++;
        }
}
print FH $tail;
close(FH);

# print domain and # of times it accessed the program
$scale = $maxsize/max_hash(\%domains);
open(FH, "> topdomains.html");
print FH $head;
print FH "<tr><th>domain</th><th>searches</th><th>graph</th></tr>\n";
my $i = 0;
foreach (sort {$domains{$b} <=> $domains{$a}} keys (%domains)) {
        if ($i < 100) {
                print FH "<tr><td>$_</td><td>".$domains{$_}."</td>
		<td><img class=\"imgleft\"  src=\"punt.png\" height=\"10\" width=\"".$domains{$_}*$scale."\"></img></td></tr>\n";
                $i++;
        }
}
print FH $tail;
close(FH);

# print hour and # of times during it the program was accessed
$scale = $maxsize/max_hash(\%hourofday);
open(FH, "> hours.html");
print FH $head;
print FH "<tr><th>hour</th><th>searches</th><th>graph</th></tr>\n";
foreach (sort {$a <=> $b} keys (%hourofday)) {
	print FH "<tr><td>$_</td><td>".$hourofday{$_}."</td>
	<td><img class=\"imgleft\"  src=\"punt.png\" height=\"10\" width=\"".$hourofday{$_}*$scale."\"></img></td></tr>\n";
}
print FH $tail;
close(FH);


# print day and # of times the program was accessed
$scale = $maxsize/max_hash(\%queriesperday);
open(FH, "> traffic.html");
print FH $head;
print FH "<tr><th>day</th><th>searches</th><th>graph</th></tr>\n";
$i = 0;
foreach (sort {$b <=> $a} keys (%queriesperday)) {
	if ($i < 100) {
	        print FH "<tr><td>$_</td><td>".$queriesperday{$_}."</td>
		<td><img class=\"imgleft\"  src=\"punt.png\" height=\"10\" width=\"".$queriesperday{$_}*$scale."\"></img></td></tr>\n";
		$i++;
	}
}
print FH $tail;
close(FH);

$scale = $maxsize/max_hash(\%queries);
open(FH, "> topqueries.html");
print FH $head;
print FH "<tr><th>query</th><th>occurences</th><th>graph</th></tr>\n";
$i = 0;
foreach (sort {$queries{$b} <=> $queries{$a}} keys (%queries)) {
	if ($i < 100) {
	        print FH "<tr><td>$_</td><td>".$queries{$_}."</td>
		<td><img class=\"imgleft\"  src=\"punt.png\" height=\"10\" width=\"".$queries{$_}*$scale."\"></img></td></tr>\n";
		$i++;
	}
}
print FH $tail;
close(FH);

my $numque = 0;
my $totaltime = 0;
$scale = $maxsize/max_hash(\%times);
open(FH, "> times.html");
print FH $head;
print FH "<tr><th>time</th><th>occurences</th><th>graph</th></tr>\n";
$i = 0;
foreach (sort {$a <=> $b} keys %times) {
	$totaltime += $times{$_}*$_;
	$numque += $times{$_};
	if ($i < 25) {
		print FH "<tr><td>$_</td><td>".$times{$_}."</td>
		<td><img class=\"imgleft\"  src=\"punt.png\" height=\"10\" width=\"".$times{$_}*$scale."\"></img></td></tr>\n";
		$i++;
	}
}
#print FH "<tr><td>average:</td><td>".($totaltime/$numque)."</td></tr>\n";

#my $average_time;$average_time = $totaltime/$numque;

print FH $tail;
close(FH);
`tail -30 queries > lastqueries`;

close(FH);

#print max_hash(\%ips); 
sub max_hash{
my $value= 0;
my $max  = 0;
my $href = shift;
my @values ;
@values= values %$href;
foreach $value (@values) {
    $max = $value if $value > $max;
}
return $max;
}


